// This file was generated by ecm_qt_declare_logging_category(): DO NOT EDIT!

#include "accessibilityinspector_debug.h"


Q_LOGGING_CATEGORY(ACCESSIBILITYINSPECTOR_LOG, "org.kde.accessibilityinspector", QtInfoMsg)

